%
% CavityTK : toolkit to built cavities using Hankel transformation
%   Longitudinal propagation phase, exp( -i k L ) is not included in the calculation.
%   That means, (k L) is interpreted as 2 n pi.
%   To adjust a microscopic displacement or to calculate sidebands, multiply appropreate phase.
%
%   needs HankelTK
%
% November 30, 2012
%   All reflections and transmissions are diag(M,0)
%
% August 19, 2010
%   all vector functions, like MirrorFrontRef, return diagonal matrix, 
%   so no need to use diag(X,0)
%
% August 2, 2010 
%   f number is replaced by transmission phase
%   phi = k r^2 / 2 f
%
% June 24, 2010 
%   Moved modal model related functions to ModalModelTK.m
%   invF can be a vector. 
%   phi (transmission phase ) = i k / 2 * invF, invF = 2 * phi(ri) / k.
%
% May 15, 2010 Hiro Yamamoto  hiro@ligo.caltech.edu
%   first release
%

classdef CavityTK < handle
    
    properties (Constant)
        lambda = 1.064e-6;
        kLam = 2*pi/CavityTK.lambda;
        nSubs = 1.44963;
    end
    
    methods (Static)
        
        % mirror front reflection
        % Eref = r * exp( i*k*(2/2ROC + 2*del) * r^2 )
        % ROC > 0 for concave mirror.
        % del is surface deformation, > 0 for convex dump
        % radius is the mirror radius
        function RefM = MirrorFrontRef( r, invROC, del, radius, hkObj )
            RefM = -r * exp( 1i*CavityTK.kLam* ...
                  ( invROC * hkObj.rList.^2 + 2*del ) ) ...
                .* CavityTK.inRadius(radius, hkObj);
            RefM = diag( RefM, 0 );
        end

        % mirror back reflection
        % Eref = r * exp( 2i*( phi - k r^2 n /2ROC ) )
        % R is the mirror radius
        function RefM = MirrorBackRef( r, invROC, phi, radius, hkObj )
            RefM = r * exp( 1i * ( 2*phi - ...
                CavityTK.kLam*CavityTK.nSubs*invROC .* hkObj.rList.^2 ) ) ...
                .* CavityTK.inRadius(radius, hkObj);
            RefM  = diag( RefM, 0 );
        end

        % mirror transmission
        % Eout = t * exp( i * ( phi - k* r^2*(n-1)/2ROC ) )
        function TraM = MirrorTrans( t, invROC, phi, radius, hkObj )
            TraM = t * exp( 1i * ( phi - ...
                CavityTK.kLam*(CavityTK.nSubs - 1)*invROC/2 .* hkObj.rList.^2 ) ) ...
                .* CavityTK.inRadius(radius, hkObj);
            TraM  = diag( TraM, 0 );
        end

        % propagator in frequency domain
        function GF = PropF( dist, hkObj )
            GF = exp( 1i*dist/(2*CavityTK.kLam) / hkObj.a^2 ...
                * hkObj.zetaList.^2 );
        end

        % propagator in x domain
        % if hkObj2 is defined, adaptive scaling is applied
        function GX = PropX( dist, hkObj1, hkObj2 )
            % if source and sink have different hkObj, apply adaptive scaling
            if nargin == 3
                alpha = hkObj2.a / hkObj1.a;
            else
                alpha = 1;
            end
            
            GX = hkObj1.HPlus \ diag(CavityTK.PropF( dist/alpha, hkObj1 ),0) * hkObj1.HPlus;
            
            if alpha ~= 1
                Cphase = CavityTK.kLam * ( alpha - 1 ) / (2*dist);
                GX = diag( exp( -1i * Cphase / alpha * hkObj2.rList.^2 ), 0 ) ...
                    * GX * diag( exp( 1i * Cphase * hkObj1.rList.^2 ), 0 ) / alpha;
            end        
        end
        
        % TEM00 by R and w
        % R > 0 means outgoing, diverging field
        function E00 = HG00RW( invR, w, hkObj )
            E00 = sqrt( 2/pi ) / w * ...
                exp( -(1/w^2 + 1i*CavityTK.kLam*invR/2) ...
                * hkObj.rList.^2 );
        end
        
        % poorman's TEM00
        % ROC1 > 0, ROC2 > 0 means a cavity with two concase mirrors
        function E00 = HG00( invROC1, invROC2, Lcav, hkObj )
            w = MMTK.w1FP( invROC1, invROC2, Lcav );
            % ROC1 > 0 means concave mirror, so use -invROC1 to make the beam converging
            E00 = CavityTK.HG00RW( -invROC1, w, hkObj );
        end
        
        % TEM00 specfified by q
        function E00 = HG00q( q, hkObj )
            [invR, w] = MMTK.q2RW( q );
            E00 = CavityTK.HG00RW( invR, w, hkObj );
        end
        
        % calculate tltal power and 00 power
        function [powerTotal, power00] = calcPower( E, E00, hkObj )
            powerTotal = hkObj.hkDotProd( E, E );
            power00 = abs( hkObj.hkDotProd( E, E00 )).^2;
        end

        % if r values are in radius R
        function inList = inRadius( radius, hkObj )
            % if radius is 0 or very large, pass all.
            if  (radius == 0) || ( radius >= max(hkObj.rList) )
                inList = 1;
            else
                inList = hkObj.rList <= radius;
            end
        end
        
    end
end




